@@ -85,6 +85,7 @@ class Cluster(object):
 
     def set_datadir_count(self, n):
         self.data_dir_count = int(n)
+        self._update_config()
         return self
 
     def set_install_dir(self, install_dir=None, version=None, verbose=False):
@@ -367,7 +368,7 @@ class Cluster(object):
 
     def start(self, no_wait=False, verbose=False, wait_for_binary_proto=True,
               wait_other_notice=True, jvm_args=None, profile_options=None,
-              quiet_start=False, allow_root=False):
+              quiet_start=False, allow_root=False, **kwargs):
         if jvm_args is None:
             jvm_args = []
 
@@ -406,7 +407,7 @@ class Cluster(object):
             for node, p, mark in started:
                 try:
                     start_message = "Listening for thrift clients..." if self.cassandra_version() < "2.2" else "Starting listening for CQL clients"
-                    node.watch_log_for(start_message, timeout=60, process=p, verbose=verbose, from_mark=mark)
+                    node.watch_log_for(start_message, timeout=kwargs.get('timeout',60), process=p, verbose=verbose, from_mark=mark)
                 except RuntimeError:
                     return None
 
@@ -483,19 +484,25 @@ class Cluster(object):
         if len(livenodes) == 0:
             print_("No live node")
             return
+        nodes_options = []
         if self.cassandra_version() <= '2.1':
-            args = [stress, '-d', ",".join(livenodes)] + stress_options
+            if '-d' not in stress_options:
+                nodes_options = ['-d', ",".join(livenodes)]
+            args = [stress] + nodes_options + stress_options
         else:
-            args = [stress] + stress_options + ['-node', ','.join(livenodes)]
+            if '-node' not in stress_options:
+                nodes_options = ['-node', ','.join(livenodes)]
+            args = [stress] + stress_options + nodes_options
+        rc = None
         try:
             # need to set working directory for env on Windows
             if common.is_win():
-                subprocess.call(args, cwd=common.parse_path(stress))
+                rc = subprocess.call(args, cwd=common.parse_path(stress))
             else:
-                subprocess.call(args)
+                rc = subprocess.call(args)
         except KeyboardInterrupt:
             pass
-        return self
+        return rc
 
     def run_cli(self, cmds=None, show_output=False, cli_options=None):
         if cli_options is None:
