@@ -60,18 +60,14 @@ NODE_CMDS = [
 ]
 
 
-def node_cmds():
+def commands():
     return NODE_CMDS
 
 
 class NodeShowCmd(Cmd):
 
-    def description(self):
-        return "Display information on a node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name show [options]"
-        return self._get_default_parser(usage, self.description())
+    descr_text = "Display information on a node"
+    usage = "usage: ccm node_name show [options]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -82,12 +78,8 @@ class NodeShowCmd(Cmd):
 
 class NodeRemoveCmd(Cmd):
 
-    def description(self):
-        return "Remove a node (stopping it if necessary and deleting all its data)"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name remove [options]"
-        return self._get_default_parser(usage, self.description())
+    descr_text = "Remove a node (stopping it if necessary and deleting all its data)"
+    usage = "usage: ccm node_name remove [options]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -98,12 +90,8 @@ class NodeRemoveCmd(Cmd):
 
 class NodeShowlogCmd(Cmd):
 
-    def description(self):
-        return "Show the log of node name (runs your $PAGER on its system.log)"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name showlog [options]"
-        return self._get_default_parser(usage, self.description())
+    descr_text = "Show the log of node name (runs your $PAGER on its system.log)"
+    usage = "usage: ccm node_name showlog [options]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -116,15 +104,11 @@ class NodeShowlogCmd(Cmd):
 
 class NodeSetlogCmd(Cmd):
 
-    def description(self):
-        return "Set node name log level (INFO, DEBUG, ...) with/without Java class - require a node restart"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name setlog [options] level"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-c', '--class', type="string", dest="class_name", default=None,
-                          help="Optional java class/package. Logging will be set for only this class/package if set")
-        return parser
+    options_list = [
+        (['-c', '--class'], {'type': "string", 'dest': "class_name", 'default': None, 'help': "Optional java class/package. Logging will be set for only this class/package if set"}),
+    ]
+    descr_text = "Set node name log level (INFO, DEBUG, ...) with/without Java class - require a node restart"
+    usage = "usage: ccm node_name setlog [options] level"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -150,15 +134,11 @@ class NodeSetlogCmd(Cmd):
 
 class NodeClearCmd(Cmd):
 
-    def description(self):
-        return "Clear the node data & logs (and stop the node)"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name_clear [options]"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-a', '--all', action="store_true", dest="all",
-                          help="Also clear the saved cache and node log files", default=False)
-        return parser
+    options_list = [
+        (['-a', '--all'], {'action': "store_true", 'dest': "all", 'help': "Also clear the saved cache and node log files", 'default': False}),
+    ]
+    descr_text = "Clear the node data & logs (and stop the node)"
+    usage = "usage: ccm node_name_clear [options]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -170,31 +150,20 @@ class NodeClearCmd(Cmd):
 
 class NodeStartCmd(Cmd):
 
-    def description(self):
-        return "Start a node"
-
-    def get_parser(self):
-        usage = "usage: ccm node start [options] name"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-v', '--verbose', action="store_true", dest="verbose",
-                          help="Print standard output of cassandra process", default=False)
-        parser.add_option('--no-wait', action="store_true", dest="no_wait",
-                          help="Do not wait for cassandra node to be ready", default=False)
-        parser.add_option('--wait-other-notice', action="store_true", dest="deprecate",
-                          help="DEPRECATED/IGNORED: Use '--skip-wait-other-notice' instead. This is now on by default.", default=False)
-        parser.add_option('--skip-wait-other-notice', action="store_false", dest="wait_other_notice",
-                          help="Skip waiting until all live nodes of the cluster have marked the other nodes UP", default=True)
-        parser.add_option('--wait-for-binary-proto', action="store_true", dest="wait_for_binary_proto",
-                          help="Wait for the binary protocol to start", default=False)
-        parser.add_option('-j', '--dont-join-ring', action="store_true", dest="no_join_ring",
-                          help="Launch the instance without joining the ring", default=False)
-        parser.add_option('--replace-address', type="string", dest="replace_address", default=None,
-                          help="Replace a node in the ring through the cassandra.replace_address option")
-        parser.add_option('--jvm_arg', action="append", dest="jvm_args",
-                          help="Specify a JVM argument", default=[])
-        parser.add_option('--quiet-windows', action="store_true", dest="quiet_start", help="Pass -q on Windows 2.2.4+ and 3.0+ startup. Ignored on linux.", default=False)
-        parser.add_option('--root', action="store_true", dest="allow_root", help="Allow CCM to start cassandra as root", default=False)
-        return parser
+    options_list = [
+        (['-v', '--verbose'], {'action': "store_true", 'dest': "verbose", 'help': "Print standard output of cassandra process", 'default': False}),
+        (['--no-wait'], {'action': "store_true", 'dest': "no_wait", 'help': "Do not wait for cassandra node to be ready", 'default': False}),
+        (['--wait-other-notice'], {'action': "store_true", 'dest': "deprecate", 'help': "DEPRECATED/IGNORED: Use '--skip-wait-other-notice' instead. This is now on by default.", 'default': False}),
+        (['--skip-wait-other-notice'], {'action': "store_false", 'dest': "wait_other_notice", 'help': "Skip waiting until all live nodes of the cluster have marked the other nodes UP", 'default': True}),
+        (['--wait-for-binary-proto'], {'action': "store_true", 'dest': "wait_for_binary_proto", 'help': "Wait for the binary protocol to start", 'default': False}),
+        (['-j', '--dont-join-ring'], {'action': "store_true", 'dest': "no_join_ring", 'help': "Launch the instance without joining the ring", 'default': False}),
+        (['--replace-address'], {'type': "string", 'dest': "replace_address", 'default': None, 'help': "Replace a node in the ring through the cassandra.replace_address option"}),
+        (['--jvm_arg'], {'action': "append", 'dest': "jvm_args", 'help': "Specify a JVM argument", 'default': []}),
+        (['--quiet-windows'], {'action': "store_true", 'dest': "quiet_start", 'help': "Pass -q on Windows 2.2.4+ and 3.0+ startup. Ignored on linux.", 'default': False}),
+        (['--root'], {'action': "store_true", 'dest': "allow_root", 'help': "Allow CCM to start cassandra as root", 'default': False}),
+    ]
+    descr_text = "Start a node"
+    usage = "usage: ccm node start [options] name"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -220,22 +189,14 @@ class NodeStartCmd(Cmd):
 
 class NodeStopCmd(Cmd):
 
-    def description(self):
-        return "Stop a node"
-
-    def get_parser(self):
-        usage = "usage: ccm node stop [options] name"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('--no-wait', action="store_true", dest="no_wait",
-                          help="Do not wait for the node to be stopped", default=False)
-        parser.add_option('-g', '--gently', action="store_const", dest="signal_event",
-                          help="Shut down gently (default)", const=signal.SIGTERM,
-                          default=signal.SIGTERM)
-        parser.add_option('--hang-up', action="store_const", dest="signal_event",
-                          help="Shut down via hang up (kill -1)", const=signal.SIGHUP)
-        parser.add_option('--not-gently', action="store_const", dest="signal_event",
-                          help="Shut down immediately (kill -9)", const=signal.SIGKILL)
-        return parser
+    options_list = [
+        (['--no-wait'], {'action': "store_true", 'dest': "no_wait", 'help': "Do not wait for the node to be stopped", 'default': False}),
+        (['-g', '--gently'], {'action': "store_const", 'dest': "signal_event", 'help': "Shut down gently (default)", 'const': signal.SIGTERM, 'default': signal.SIGTERM}),
+        (['--hang-up'], {'action': "store_const", 'dest': "signal_event", 'help': "Shut down via hang up (kill -1)", 'const': common.get_default_signals()['1']}),
+        (['--not-gently'], {'action': "store_const", 'dest': "signal_event", 'help': "Shut down immediately (kill -9)", 'const': common.get_default_signals()['9']}),
+    ]
+    descr_text = "Stop a node"
+    usage = "usage: ccm node stop [options] name"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -255,13 +216,6 @@ class _NodeToolCmd(Cmd):
     descr_text = "This is a private class, how did you get here?"
     nodetool_cmd = ''
 
-    def get_parser(self):
-        parser = self._get_default_parser(self.usage, self.description())
-        return parser
-
-    def description(self):
-        return self.descr_text
-
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
 
@@ -331,15 +285,12 @@ class NodeVersionCmd(_NodeToolCmd):
 
 class NodeDecommissionCmd(_NodeToolCmd):
     usage = "usage: ccm node_name decommission [options]"
+    options_list = [
+        (['--force'], {'action': "store_true", 'dest': "force", 'help': "Force decommission of this node even when it reduces the number of replicas to below configured RF.  Note: This is only relevant for C* 3.12+.", 'default': False}),
+    ]
     nodetool_cmd = 'decommission'
     descr_text = "Run decommission on node name"
 
-    def get_parser(self):
-        parser = self._get_default_parser(self.usage, self.description())
-        parser.add_option('--force', action="store_true", dest="force",
-                help="Force decommission of this node even when it reduces the number of replicas to below configured RF.  Note: This is only relevant for C* 3.12+.", default=False)
-        return parser
-
     def run(self):
         self.node.decommission(force=self.options.force)
 
@@ -349,18 +300,13 @@ class _DseToolCmd(Cmd):
     descr_text = "This is a private class, how did you get here?"
     dsetool_cmd = ''
 
-    def get_parser(self):
-        parser = self._get_default_parser(self.usage, self.description())
-        return parser
-
-    def description(self):
-        return self.descr_text
-
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
 
     def run(self):
-        self.node.dsetool(self.dsetool_cmd)
+        stdout, stderr, rc = self.node.dsetool(self.dsetool_cmd + " " + " ".join((self.args[1:])))
+        print_(stderr)
+        print_(stdout)
 
 
 class NodeDsetoolCmd(_DseToolCmd):
@@ -368,22 +314,20 @@ class NodeDsetoolCmd(_DseToolCmd):
     descr_text = "Run dsetool (connecting to node name)"
 
     def run(self):
-        self.node.dsetool(" ".join(self.args[1:]))
+        stdout, stderr, rc = self.node.dsetool(" ".join(self.args[1:]))
+        print_(stderr)
+        print_(stdout)
 
 
 class NodeCliCmd(Cmd):
 
-    def description(self):
-        return "Launch a cassandra cli connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name cli [options] [cli_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        parser.add_option('-x', '--exec', type="string", dest="cmds", default=None,
-                          help="Execute the specified commands and exit")
-        parser.add_option('-v', '--verbose', action="store_true", dest="verbose",
-                          help="With --exec, show cli output after completion", default=False)
-        return parser
+    options_list = [
+        (['-x', '--exec'], {'type': "string", 'dest': "cmds", 'default': None, 'help': "Execute the specified commands and exit"}),
+        (['-v', '--verbose'], {'action': "store_true", 'dest': "verbose", 'help': "With --exec, show cli output after completion", 'default': False}),
+    ]
+    descr_text = "Launch a cassandra cli connected to this node"
+    usage = "usage: ccm node_name cli [options] [cli_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -400,17 +344,13 @@ class NodeCliCmd(Cmd):
 
 class NodeCqlshCmd(Cmd):
 
-    def description(self):
-        return "Launch a cqlsh session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name cqlsh [options] [cli_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        parser.add_option('-x', '--exec', type="string", dest="cmds", default=None,
-                          help="Execute the specified commands and exit")
-        parser.add_option('-v', '--verbose', action="store_true", dest="verbose",
-                          help="With --exec, show cli output after completion", default=False)
-        return parser
+    options_list = [
+        (['-x', '--exec'], {'type': "string", 'dest': "cmds", 'default': None, 'help': "Execute the specified commands and exit"}),
+        (['-v', '--verbose'], {'action': "store_true", 'dest': "verbose", 'help': "With --exec, show cli output after completion", 'default': False}),
+    ]
+    descr_text = "Launch a cqlsh session connected to this node"
+    usage = "usage: ccm node_name cqlsh [options] [cli_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -422,13 +362,9 @@ class NodeCqlshCmd(Cmd):
 
 class NodeBulkloadCmd(Cmd):
 
-    def description(self):
-        return "Bulkload files into the cluster by connecting to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name bulkload [options] [sstable_dir]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Bulkload files into the cluster by connecting to this node"
+    usage = "usage: ccm node_name bulkload [options] [sstable_dir]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -440,13 +376,9 @@ class NodeBulkloadCmd(Cmd):
 
 class NodeScrubCmd(Cmd):
 
-    def description(self):
-        return "Scrub files"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name scrub [options] <keyspace> <cf>"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Scrub files"
+    usage = "usage: ccm node_name scrub [options] <keyspace> <cf>"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -458,13 +390,9 @@ class NodeScrubCmd(Cmd):
 
 class NodeVerifyCmd(Cmd):
 
-    def description(self):
-        return "Verify files"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name verify [options] <keyspace> <cf>"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Verify files"
+    usage = "usage: ccm node_name verify [options] <keyspace> <cf>"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -476,21 +404,14 @@ class NodeVerifyCmd(Cmd):
 
 class NodeJsonCmd(Cmd):
 
-    def description(self):
-        return "Call sstable2json/sstabledump on the sstables of this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name json [options] [file]"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-k', '--keyspace', type="string", dest="keyspace", default=None,
-                          help="The keyspace to use [use all keyspaces by default]")
-        parser.add_option('-c', '--column-families', type="string", dest="cfs", default=None,
-                          help="Comma separated list of column families to use (requires -k to be set)")
-        parser.add_option('--key', type="string", action="append", dest="keys", default=None,
-                          help="The key to include (you may specify multiple --key)")
-        parser.add_option('-e', '--enumerate-keys', action="store_true", dest="enumerate_keys",
-                          help="Only enumerate keys (i.e, call sstable2keys)", default=False)
-        return parser
+    options_list = [
+        (['-k', '--keyspace'], {'type': "string", 'dest': "keyspace", 'default': None, 'help': "The keyspace to use [use all keyspaces by default]"}),
+        (['-c', '--column-families'], {'type': "string", 'dest': "cfs", 'default': None, 'help': "Comma separated list of column families to use (requires -k to be set)"}),
+        (['--key'], {'type': "string", 'action': "append", 'dest': "keys", 'default': None, 'help': "The key to include (you may specify multiple --key)"}),
+        (['-e', '--enumerate-keys'], {'action': "store_true", 'dest': "enumerate_keys", 'help': "Only enumerate keys (i.e, call sstable2keys)", 'default': False}),
+    ]
+    descr_text = "Call sstable2json/sstabledump on the sstables of this node"
+    usage = "usage: ccm node_name json [options] [file]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -515,30 +436,24 @@ class NodeJsonCmd(Cmd):
                                            enumerate_keys=self.options.enumerate_keys)
             elif self.node.has_cmd('sstabledump'):
                 self.node.run_sstabledump(keyspace=self.keyspace,
-                                           column_families=self.column_families,
-                                           keys=self.options.keys,
-                                           enumerate_keys=self.options.enumerate_keys)
+                                          column_families=self.column_families,
+                                          keys=self.options.keys,
+                                          enumerate_keys=self.options.enumerate_keys,
+                                          command=True)
         except common.ArgumentError as e:
             print_(e, file=sys.stderr)
 
 
 class NodeSstablesplitCmd(Cmd):
 
-    def description(self):
-        return "Run sstablesplit on the sstables of this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name sstablesplit [options] [file]"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-k', '--keyspace', type="string", dest="keyspace", default=None,
-                          help="The keyspace to use [use all keyspaces by default]")
-        parser.add_option('-c', '--column-families', type="string", dest='cfs', default=None,
-                          help="Comma separated list of column families to use (requires -k to be set)")
-        parser.add_option('-s', '--size', type='int', dest="size", default=None,
-                          help="Maximum size in MB for the output sstables (default: 50 MB)")
-        parser.add_option('--no-snapshot', action='store_true', dest="no_snapshot", default=False,
-                          help="Don't snapshot the sstables before splitting")
-        return parser
+    options_list = [
+        (['-k', '--keyspace'], {'type': "string", 'dest': "keyspace", 'default': None, 'help': "The keyspace to use [use all keyspaces by default]"}),
+        (['-c', '--column-families'], {'type': "string", 'dest': 'cfs', 'default': None, 'help': "Comma separated list of column families to use (requires -k to be set)"}),
+        (['-s', '--size'], {'type': 'int', 'dest': "size", 'default': None, 'help': "Maximum size in MB for the output sstables (default: 50 MB)"}),
+        (['--no-snapshot'], {'action': 'store_true', 'dest': "no_snapshot", 'default': False, 'help': "Don't snapshot the sstables before splitting"}),
+    ]
+    descr_text = "Run sstablesplit on the sstables of this node"
+    usage = "usage: ccm node_name sstablesplit [options] [file]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -567,17 +482,12 @@ class NodeSstablesplitCmd(Cmd):
 
 class NodeGetsstablesCmd(Cmd):
 
-    def description(self):
-        return "Run getsstables to get absolute path of sstables in this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name getsstables [options] [file]"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-k', '--keyspace', type="string", dest="keyspace", default=None,
-                          help="The keyspace to use [use all keyspaces by default]")
-        parser.add_option('-t', '--tables', type="string", dest='tables', default=None,
-                          help="Comma separated list of tables to use (requires -k to be set)")
-        return parser
+    options_list = [
+        (['-k', '--keyspace'], {'type': "string", 'dest': "keyspace", 'default': None, 'help': "The keyspace to use [use all keyspaces by default]"}),
+        (['-t', '--tables'], {'type': "string", 'dest': 'tables', 'default': None, 'help': "Comma separated list of tables to use (requires -k to be set)"}),
+    ]
+    descr_text = "Run getsstables to get absolute path of sstables in this node"
+    usage = "usage: ccm node_name getsstables [options] [file]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -604,23 +514,15 @@ class NodeGetsstablesCmd(Cmd):
 
 class NodeUpdateconfCmd(Cmd):
 
-    def description(self):
-        return "Update the cassandra config files for this node (useful when updating cassandra)"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name updateconf [options] [ new_setting | ...  ], where new_setting should be a string of the form 'compaction_throughput_mb_per_sec: 32'"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('--no-hh', '--no-hinted-handoff', action="store_false",
-                          dest="hinted_handoff", default=True, help="Disable hinted handoff")
-        parser.add_option('--batch-cl', '--batch-commit-log', action="store_true",
-                          dest="cl_batch", default=None, help="Set commit log to batch mode")
-        parser.add_option('--periodic-cl', '--periodic-commit-log', action="store_true",
-                          dest="cl_periodic", default=None, help="Set commit log to periodic mode")
-        parser.add_option('--rt', '--rpc-timeout', action="store", type='int',
-                          dest="rpc_timeout", help="Set rpc timeout")
-        parser.add_option('-y', '--yaml', action="store_true", dest="literal_yaml",
-                          default=False, help="Pass in literal yaml string. Option syntax looks like ccm node_name updateconf -y 'a: [b: [c,d]]'")
-        return parser
+    options_list = [
+        (['--no-hh', '--no-hinted-handoff'], {'action': "store_false", 'dest': "hinted_handoff", 'default': True, 'help': "Disable hinted handoff"}),
+        (['--batch-cl', '--batch-commit-log'], {'action': "store_true", 'dest': "cl_batch", 'default': None, 'help': "Set commit log to batch mode"}),
+        (['--periodic-cl', '--periodic-commit-log'], {'action': "store_true", 'dest': "cl_periodic", 'default': None, 'help': "Set commit log to periodic mode"}),
+        (['--rt', '--rpc-timeout'], {'action': "store", 'type': 'int', 'dest': "rpc_timeout", 'help': "Set rpc timeout"}),
+        (['-y', '--yaml'], {'action': "store_true", 'dest': "literal_yaml", 'default': False, 'help': "Pass in literal yaml string. Option syntax looks like ccm node_name updateconf -y 'a: [b: [c,d]]'"}),
+    ]
+    descr_text = "Update the cassandra config files for this node (useful when updating cassandra)"
+    usage = "usage: ccm node_name updateconf [options] [ new_setting | ...  ], where new_setting should be a string of the form 'compaction_throughput_mb_per_sec: 32'"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -655,14 +557,11 @@ class NodeUpdateconfCmd(Cmd):
 
 class NodeUpdatedseconfCmd(Cmd):
 
-    def description(self):
-        return "Update the dse config files for this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name updatedseconf [options] [ new_setting | ...  ], where new setting should be a string of the form 'max_solr_concurrency_per_core: 2'; nested options can be separated with a period like 'cql_slow_log_options.enabled: true'"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-y', '--yaml', action="store_true", dest="literal_yaml", default=False, help="Pass in literal yaml string. Option syntax looks like ccm node_name updatedseconf -y 'a: [b: [c,d]]'")
-        return parser
+    options_list = [
+        (['-y', '--yaml'], {'action': "store_true", 'dest': "literal_yaml", 'default': False, 'help': "Pass in literal yaml string. Option syntax looks like ccm node_name updatedseconf -y 'a: [b: [c,d]]'"}),
+    ]
+    descr_text = "Update the dse config files for this node"
+    usage = "usage: ccm node_name updatedseconf [options] [ new_setting | ...  ], where new setting should be a string of the form 'max_solr_concurrency_per_core: 2'; nested options can be separated with a period like 'cql_slow_log_options.enabled: true'"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -685,15 +584,11 @@ class NodeUpdatedseconfCmd(Cmd):
 
 class NodeUpdatelog4jCmd(Cmd):
 
-    def description(self):
-        return "Update the Cassandra log4j-server.properties configuration file under given node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name updatelog4j -p <log4j config>"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-p', '--path', type="string", dest="log4jpath",
-                          help="Path to new Cassandra log4j configuration file")
-        return parser
+    options_list = [
+        (['-p', '--path'], {'type': "string", 'dest': "log4jpath", 'help': "Path to new Cassandra log4j configuration file"}),
+    ]
+    descr_text = "Update the Cassandra log4j-server.properties configuration file under given node"
+    usage = "usage: ccm node_name updatelog4j -p <log4j config>"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -718,13 +613,9 @@ class NodeUpdatelog4jCmd(Cmd):
 
 class NodeStressCmd(Cmd):
 
-    def description(self):
-        return "Run stress on a node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name stress [options] [stress_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Run stress on a node"
+    usage = "usage: ccm node_name stress [options] [stress_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -739,13 +630,9 @@ class NodeStressCmd(Cmd):
 
 class NodeShuffleCmd(Cmd):
 
-    def description(self):
-        return "Run shuffle on a node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name shuffle [options] [shuffle_cmds]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Run shuffle on a node"
+    usage = "usage: ccm node_name shuffle [options] [shuffle_cmds]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -757,17 +644,12 @@ class NodeShuffleCmd(Cmd):
 
 class NodeSetdirCmd(Cmd):
 
-    def description(self):
-        return "Set the cassandra directory to use for the node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name setdir [options]"
-        parser = self._get_default_parser(usage, self.description())
-        parser.add_option('-v', "--version", type="string", dest="version",
-                          help="Download and use provided cassandra or dse version. If version is of the form 'git:<branch name>', then the specified branch will be downloaded from the git repo and compiled. (takes precedence over --install-dir)", default=None)
-        parser.add_option("--install-dir", type="string", dest="install_dir",
-                          help="Path to the cassandra or dse directory to use [default %default]", default="./")
-        return parser
+    options_list = [
+        (['-v', "--version"], {'type': "string", 'dest': "version", 'help': "Download and use provided cassandra or dse version. If version is of the form 'git:<branch name>', then the specified branch will be downloaded from the git repo and compiled. (takes precedence over --install-dir)", 'default': None}),
+        (["--install-dir"], {'type': "string", 'dest': "install_dir", 'help': "Path to the cassandra or dse directory to use [default %default]", 'default': "./"}),
+    ]
+    descr_text = "Set the cassandra directory to use for the node"
+    usage = "usage: ccm node_name setdir [options]"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -782,13 +664,8 @@ class NodeSetdirCmd(Cmd):
 
 class NodeSetworkloadCmd(Cmd):
 
-    def description(self):
-        return "Sets the workloads for a DSE node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name setworkload [cassandra|solr|hadoop|spark|dsefs|cfs|graph],..."
-        parser = self._get_default_parser(usage, self.description())
-        return parser
+    descr_text = "Sets the workloads for a DSE node"
+    usage = "usage: ccm node_name setworkload [cassandra|solr|hadoop|spark|dsefs|cfs|graph],..."
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -809,13 +686,9 @@ class NodeSetworkloadCmd(Cmd):
 
 class NodeDseCmd(Cmd):
 
-    def description(self):
-        return "Launch a dse client application connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name dse [dse_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a dse client application connected to this node"
+    usage = "usage: ccm node_name dse [dse_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -827,13 +700,9 @@ class NodeDseCmd(Cmd):
 
 class NodeHadoopCmd(Cmd):
 
-    def description(self):
-        return "Launch a hadoop session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name hadoop [options] [hadoop_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a hadoop session connected to this node"
+    usage = "usage: ccm node_name hadoop [options] [hadoop_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -845,13 +714,9 @@ class NodeHadoopCmd(Cmd):
 
 class NodeHiveCmd(Cmd):
 
-    def description(self):
-        return "Launch a hive session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name hive [options] [hive_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a hive session connected to this node"
+    usage = "usage: ccm node_name hive [options] [hive_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -863,13 +728,9 @@ class NodeHiveCmd(Cmd):
 
 class NodePigCmd(Cmd):
 
-    def description(self):
-        return "Launch a pig session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name pig [options] [pig_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a pig session connected to this node"
+    usage = "usage: ccm node_name pig [options] [pig_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -881,13 +742,9 @@ class NodePigCmd(Cmd):
 
 class NodeSqoopCmd(Cmd):
 
-    def description(self):
-        return "Launch a sqoop session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name sqoop [options] [sqoop_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a sqoop session connected to this node"
+    usage = "usage: ccm node_name sqoop [options] [sqoop_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -899,13 +756,9 @@ class NodeSqoopCmd(Cmd):
 
 class NodeSparkCmd(Cmd):
 
-    def description(self):
-        return "Launch a spark session connected to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name spark [options] [spark_options]"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Launch a spark session connected to this node"
+    usage = "usage: ccm node_name spark [options] [spark_options]"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -917,13 +770,8 @@ class NodeSparkCmd(Cmd):
 
 class NodePauseCmd(Cmd):
 
-    def description(self):
-        return "Send a SIGSTOP to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name pause"
-        parser = self._get_default_parser(usage, self.description())
-        return parser
+    descr_text = "Send a SIGSTOP to this node"
+    usage = "usage: ccm node_name pause"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -934,13 +782,8 @@ class NodePauseCmd(Cmd):
 
 class NodeResumeCmd(Cmd):
 
-    def description(self):
-        return "Send a SIGCONT to this node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name resume"
-        parser = self._get_default_parser(usage, self.description())
-        return parser
+    descr_text = "Send a SIGCONT to this node"
+    usage = "usage: ccm node_name resume"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -951,12 +794,8 @@ class NodeResumeCmd(Cmd):
 
 class NodeJconsoleCmd(Cmd):
 
-    def description(self):
-        return "Opens jconsole client and connect to running node"
-
-    def get_parser(self):
-        usage = "usage: ccm node_name jconsole"
-        return self._get_default_parser(usage, self.description())
+    descr_text = "Opens jconsole client and connect to running node"
+    usage = "usage: ccm node_name jconsole"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -972,13 +811,8 @@ class NodeJconsoleCmd(Cmd):
 
 class NodeVersionfrombuildCmd(Cmd):
 
-    def description(self):
-        return "Print the node's version as grepped from build.xml. Can be used when the node isn't running."
-
-    def get_parser(self):
-        usage = "usage: ccm node_name versionfrombuild"
-        parser = self._get_default_parser(usage, self.description())
-        return parser
+    descr_text = "Print the node's version as grepped from build.xml. Can be used when the node isn't running."
+    usage = "usage: ccm node_name versionfrombuild"
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
@@ -989,13 +823,9 @@ class NodeVersionfrombuildCmd(Cmd):
 
 class NodeBytemanCmd(Cmd):
 
-    def description(self):
-        return "Invoke byteman-submit "
-
-    def get_parser(self):
-        usage = "usage: ccm node_name byteman-submit"
-        parser = self._get_default_parser(usage, self.description(), ignore_unknown_options=True)
-        return parser
+    descr_text = "Invoke byteman-submit "
+    usage = "usage: ccm node_name byteman-submit"
+    ignore_unknown_options = True
 
     def validate(self, parser, options, args):
         Cmd.validate(self, parser, options, args, node_name=True, load_cluster=True)
