@@ -11,6 +11,7 @@ import os
 import platform
 import re
 import shutil
+import signal
 import socket
 import stat
 import subprocess
@@ -39,12 +40,27 @@ CASSANDRA_SH = "cassandra.in.sh"
 CONFIG_FILE = "config"
 CCM_CONFIG_DIR = "CCM_CONFIG_DIR"
 
-logging.basicConfig(format='%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s',
-                    datefmt='%H:%M:%S',
-                    level=logging.DEBUG
-                    )
+class InfoFilter(logging.Filter):
+    def filter(self, rec):
+        return rec.levelno in (logging.DEBUG, logging.INFO)
+
+LOG_FMT = '%(asctime)s,%(msecs)d %(name)s %(levelname)s %(message)s'
+DATE_FMT = '%H:%M:%S'
+FORMATTER = logging.Formatter(LOG_FMT, DATE_FMT)
+
+INFO_HANDLER = logging.StreamHandler(sys.stdout)
+INFO_HANDLER.setLevel(logging.DEBUG)
+INFO_HANDLER.addFilter(InfoFilter())
+INFO_HANDLER.setFormatter(FORMATTER)
+
+ERROR_HANDLER = logging.StreamHandler(sys.stderr)
+ERROR_HANDLER.setLevel(logging.WARNING)
+ERROR_HANDLER.setFormatter(FORMATTER)
 
 LOG = logging.getLogger('ccm')
+LOG.setLevel(logging.DEBUG)
+LOG.addHandler(INFO_HANDLER)
+LOG.addHandler(ERROR_HANDLER)
 
 
 def error(msg):
@@ -78,11 +94,13 @@ class ArgumentError(CCMError):
 class UnavailableSocketError(CCMError):
     pass
 
+
 class TimeoutError(Exception):
 
     def __init__(self, data):
         Exception.__init__(self, str(data))
 
+
 class LogPatternToVersion(object):
 
     def __init__(self, versions_to_patterns, default_pattern=None):
@@ -189,7 +207,7 @@ def replace_in_file(file, regexp, replace):
 
 def replaces_in_file(file, replacement_list):
     rs = [(re.compile(regexp), repl) for (regexp, repl) in replacement_list]
-    file_tmp = file + ".tmp"
+    file_tmp = file + "." + str(os.getpid()) + ".tmp"
     with open(file, 'r') as f:
         with open(file_tmp, 'w') as f_tmp:
             for line in f:
@@ -208,7 +226,7 @@ def replace_or_add_into_file_tail(file, regexp, replace):
 def replaces_or_add_into_file_tail(file, replacement_list, add_config_close=True):
     rs = [(re.compile(regexp), repl) for (regexp, repl) in replacement_list]
     is_line_found = False
-    file_tmp = file + ".tmp"
+    file_tmp = file + "." + str(os.getpid()) + ".tmp"
     with open(file, 'r') as f:
         with open(file_tmp, 'w') as f_tmp:
             for line in f:
@@ -246,7 +264,7 @@ def make_cassandra_env(install_dir, node_path, update_conf=True):
         sh_file = os.path.join(BIN_DIR, CASSANDRA_SH)
     orig = os.path.join(install_dir, sh_file)
     dst = os.path.join(node_path, sh_file)
-    if not os.path.exists(dst):
+    if not is_win() or not os.path.exists(dst):
         shutil.copy(orig, dst)
 
     if update_conf and not (is_win() and get_version_from_build(node_path=node_path) >= '2.1'):
@@ -277,11 +295,13 @@ def make_cassandra_env(install_dir, node_path, update_conf=True):
 
 
 def make_dse_env(install_dir, node_path, node_ip):
+    version = get_version_from_build(node_path=node_path)
     env = os.environ.copy()
     env['MAX_HEAP_SIZE'] = os.environ.get('CCM_MAX_HEAP_SIZE', '500M')
     env['HEAP_NEWSIZE'] = os.environ.get('CCM_HEAP_NEWSIZE', '50M')
-    env['SPARK_WORKER_MEMORY'] = os.environ.get('SPARK_WORKER_MEMORY', '1024M')
-    env['SPARK_WORKER_CORES'] = os.environ.get('SPARK_WORKER_CORES', '2')
+    if version < '6.0':
+        env['SPARK_WORKER_MEMORY'] = os.environ.get('SPARK_WORKER_MEMORY', '1024M')
+        env['SPARK_WORKER_CORES'] = os.environ.get('SPARK_WORKER_CORES', '2')
     env['DSE_HOME'] = os.path.join(install_dir)
     env['DSE_CONF'] = os.path.join(node_path, 'resources', 'dse', 'conf')
     env['CASSANDRA_HOME'] = os.path.join(install_dir, 'resources', 'cassandra')
@@ -302,7 +322,7 @@ def make_dse_env(install_dir, node_path, node_ip):
     env['DSE_LOG_ROOT'] = os.path.join(node_path, 'logs', 'dse')
     env['CASSANDRA_LOG_DIR'] = os.path.join(node_path, 'logs')
     env['SPARK_LOCAL_IP'] = '' + node_ip
-    if get_version_from_build(node_path=node_path) >= '5.0':
+    if version >= '5.0':
         env['HADOOP1_CONF_DIR'] = os.path.join(node_path, 'resources', 'hadoop', 'conf')
         env['HADOOP2_CONF_DIR'] = os.path.join(node_path, 'resources', 'hadoop2-client', 'conf')
     else:
@@ -507,6 +527,8 @@ def check_socket_listening(itf, timeout=60):
             sock.close()
             return True
         except socket.error:
+            if sock:
+                sock.close()
             # Try again in another 200ms
             time.sleep(.2)
             continue
@@ -630,7 +652,7 @@ def get_version_from_build(install_dir=None, node_path=None):
 def get_dse_version(install_dir):
     for root, dirs, files in os.walk(install_dir):
         for file in files:
-            match = re.search('^dse(?:-core)?-([0-9.]+)(?:-SNAPSHOT)?\.jar', file)
+            match = re.search('^dse(?:-core)?-([0-9.]+)(?:-.*)?\.jar', file)
             if match:
                 return match.group(1)
     return None
@@ -696,7 +718,7 @@ def merge_configuration(original, changes, delete_empty=True):
         for k, v in changes.items():
             # If the new value is None or an empty string, delete it
             # if it's in the original data.
-            if delete_empty and k in new and \
+            if delete_empty and k in new and new[k] is not None and \
                     (v is None or (isinstance(v, str) and len(v) == 0)):
                 del new[k]
             else:
@@ -740,3 +762,13 @@ def wait_for_any_log(nodes, pattern, timeout, filename='system.log'):
 
     raise TimeoutError(time.strftime("%d %b %Y %H:%M:%S", time.gmtime()) +
                        " Unable to find: " + repr(pattern) + " in any node log within " + str(timeout) + "s")
+
+
+def get_default_signals():
+    if is_win():
+        # Fill the dictionary with SIGTERM as the cluster is killed forcefully
+        # on Windows regardless of assigned signal (TASKKILL is used)
+        default_signal_events = {'1': signal.SIGTERM, '9': signal.SIGTERM}
+    else:
+        default_signal_events = {'1': signal.SIGHUP, '9': signal.SIGKILL}
+    return default_signal_events
