@@ -12,8 +12,9 @@ import time
 import yaml
 from six import iteritems, print_
 
-from ccmlib import common, extension
-from ccmlib.node import Node, NodeError, ToolError
+from ccmlib import common, extension, repository
+from ccmlib.node import (Node, NodeError, ToolError,
+                         handle_external_tool_process)
 
 
 class DseNode(Node):
@@ -48,32 +49,36 @@ class DseNode(Node):
         return [common.join_bin(os.path.join(self.get_install_dir(), 'resources', 'cassandra'), 'bin', 'dse'), toolname]
 
     def get_env(self):
-        (node_ip, _) = self.network_interfaces['binary']
-        return common.make_dse_env(self.get_install_dir(), self.get_path(), node_ip)
+        return common.make_dse_env(self.get_install_dir(), self.get_path(), self.ip_addr)
 
     def get_cassandra_version(self):
         return common.get_dse_cassandra_version(self.get_install_dir())
 
+    def node_setup(self, version, verbose):
+        dir, v = repository.setup_dse(version, self.cluster.dse_username, self.cluster.dse_password, verbose=verbose)
+        return dir
+
     def set_workloads(self, workloads):
         self.workloads = workloads
         self._update_config()
         if 'solr' in self.workloads:
             self.__generate_server_xml()
         if 'graph' in self.workloads:
-            (node_ip, _) = self.network_interfaces['binary']
-            conf_file = os.path.join(self.get_path(), 'resources', 'dse', 'conf', 'dse.yaml')
-            with open(conf_file, 'r') as f:
-                data = yaml.load(f)
-            graph_options = data['graph']
-            graph_options['gremlin_server']['host'] = node_ip
-            self.set_dse_configuration_options({'graph': graph_options})
             self.__update_gremlin_config_yaml()
         if 'dsefs' in self.workloads:
-            dsefs_options = {'dsefs_options': {'enabled': 'true',
+            dsefs_options = {'dsefs_options': {'enabled': True,
                                                'work_dir': os.path.join(self.get_path(), 'dsefs'),
                                                'data_directories': [{'dir': os.path.join(self.get_path(), 'dsefs', 'data')}]}}
             self.set_dse_configuration_options(dsefs_options)
         if 'spark' in self.workloads:
+            dsefs_enabled = 'dsefs' in self.workloads
+            dse_options = {'dsefs_options': {'enabled': dsefs_enabled,
+                                             'work_dir': os.path.join(self.get_path(), 'dsefs'),
+                                             'data_directories': [{'dir': os.path.join(self.get_path(), 'dsefs', 'data')}]}}
+            if self.cluster.version() >= '6.0':
+                dse_options['resource_manager_options'] = {'worker_options' : {'memory_total': '1g', 'cores_total': 2}}
+
+            self.set_dse_configuration_options(dse_options)
             self._update_spark_env()
 
     def set_dse_configuration_options(self, values=None):
@@ -149,16 +154,16 @@ class DseNode(Node):
         agent_dir = os.path.join(self.get_path(), 'datastax-agent')
         if os.path.exists(agent_dir):
             pidfile = os.path.join(agent_dir, 'datastax-agent.pid')
-        if os.path.exists(pidfile):
-            with open(pidfile, 'r') as f:
-                pid = int(f.readline().strip())
-                f.close()
-            if pid is not None:
-                try:
-                    os.kill(pid, signal.SIGKILL)
-                except OSError:
-                    pass
-            os.remove(pidfile)
+            if os.path.exists(pidfile):
+                with open(pidfile, 'r') as f:
+                    pid = int(f.readline().strip())
+                    f.close()
+                if pid is not None:
+                    try:
+                        os.kill(pid, signal.SIGKILL)
+                    except OSError:
+                        pass
+                os.remove(pidfile)
 
     def nodetool(self, cmd, username=None, password=None, capture_output=True, wait=True):
         if password is not None:
@@ -171,11 +176,12 @@ class DseNode(Node):
     def dsetool(self, cmd):
         env = self.get_env()
         extension.append_to_client_env(self, env)
+        node_ip, binary_port = self.network_interfaces['binary']
         dsetool = common.join_bin(self.get_install_dir(), 'bin', 'dsetool')
-        args = [dsetool, '-h', 'localhost', '-j', str(self.jmx_port)]
+        args = [dsetool, '-h', node_ip, '-j', str(self.jmx_port), '-c', str(binary_port)]
         args += cmd.split()
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        return handle_external_tool_process(p, args)
 
     def dse(self, dse_options=None):
         if dse_options is None:
@@ -186,8 +192,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse]
         args += dse_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def hadoop(self, hadoop_options=None):
         if hadoop_options is None:
@@ -197,8 +203,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse, 'hadoop']
         args += hadoop_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def hive(self, hive_options=None):
         if hive_options is None:
@@ -208,8 +214,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse, 'hive']
         args += hive_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def pig(self, pig_options=None):
         if pig_options is None:
@@ -219,8 +225,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse, 'pig']
         args += pig_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def sqoop(self, sqoop_options=None):
         if sqoop_options is None:
@@ -230,8 +236,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse, 'sqoop']
         args += sqoop_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def spark(self, spark_options=None):
         if spark_options is None:
@@ -241,8 +247,8 @@ class DseNode(Node):
         dse = common.join_bin(self.get_install_dir(), 'bin', 'dse')
         args = [dse, 'spark']
         args += spark_options
-        p = subprocess.Popen(args, env=env)
-        p.wait()
+        p = subprocess.Popen(args, env=env)  #Don't redirect stdout/stderr, users need to interact with new process
+        return handle_external_tool_process(p, args)
 
     def import_dse_config_files(self):
         self._update_config()
@@ -286,11 +292,15 @@ class DseNode(Node):
             shutil.copytree(src_lib, dst_lib)
 
     def import_bin_files(self):
+        common.copy_directory(os.path.join(self.get_install_dir(), 'bin'), self.get_bin_dir())
         cassandra_bin_dir = os.path.join(self.get_path(), 'resources', 'cassandra', 'bin')
         shutil.rmtree(cassandra_bin_dir, ignore_errors=True)
         os.makedirs(cassandra_bin_dir)
-        common.copy_directory(os.path.join(self.get_install_dir(), 'bin'), self.get_bin_dir())
         common.copy_directory(os.path.join(self.get_install_dir(), 'resources', 'cassandra', 'bin'), cassandra_bin_dir)
+        if os.path.exists(os.path.join(self.get_install_dir(), 'resources', 'cassandra', 'tools')):
+            cassandra_tools_dir = os.path.join(self.get_path(), 'resources', 'cassandra', 'tools')
+            shutil.rmtree(cassandra_tools_dir, ignore_errors=True)
+            shutil.copytree(os.path.join(self.get_install_dir(), 'resources', 'cassandra', 'tools'), cassandra_tools_dir)
         self.export_dse_home_in_dse_env_sh()
 
     def export_dse_home_in_dse_env_sh(self):
@@ -362,7 +372,7 @@ class DseNode(Node):
         with open(server_xml, 'w+') as f:
             f.write('<Server port="8005" shutdown="SHUTDOWN">\n')
             f.write('  <Service name="Solr">\n')
-            f.write('    <Connector port="8983" address="%s" protocol="HTTP/1.1" connectionTimeout="20000" maxThreads = "200" URIEncoding="UTF-8"/>\n' % self.network_interfaces['thrift'][0])
+            f.write('    <Connector port="8983" address="%s" protocol="HTTP/1.1" connectionTimeout="20000" maxThreads = "200" URIEncoding="UTF-8"/>\n' % self.ip_addr)
             f.write('    <Engine name="Solr" defaultHost="localhost">\n')
             f.write('      <Host name="localhost"  appBase="../solr/web"\n')
             f.write('            unpackWARs="true" autoDeploy="true"\n')
@@ -374,13 +384,11 @@ class DseNode(Node):
             f.close()
 
     def __update_gremlin_config_yaml(self):
-        (node_ip, _) = self.network_interfaces['binary']
-
         conf_file = os.path.join(self.get_path(), 'resources', 'graph', 'gremlin-console', 'conf', 'remote.yaml')
         with open(conf_file, 'r') as f:
             data = yaml.load(f)
 
-        data['hosts'] = [node_ip]
+        data['hosts'] = [self.ip_addr]
 
         with open(conf_file, 'w') as f:
             yaml.safe_dump(data, f, default_flow_style=False)
@@ -401,7 +409,7 @@ class DseNode(Node):
         address_yaml = os.path.join(agent_dir, 'conf', 'address.yaml')
         if not os.path.exists(address_yaml):
             with open(address_yaml, 'w+') as f:
-                (ip, port) = self.network_interfaces['thrift']
+                ip = self.ip_addr
                 jmx = self.jmx_port
                 f.write('stomp_interface: 127.0.0.1\n')
                 f.write('local_interface: %s\n' % ip)
@@ -410,7 +418,10 @@ class DseNode(Node):
                 f.write('cassandra_conf: %s\n' % os.path.join(self.get_path(), 'resources', 'cassandra', 'conf', 'cassandra.yaml'))
                 f.write('cassandra_install: %s\n' % self.get_path())
                 f.write('cassandra_logs: %s\n' % os.path.join(self.get_path(), 'logs'))
-                f.write('thrift_port: %s\n' % port)
+                if 'thrift' in self.network_interfaces: 
+                    (_, port) = self.network_interfaces['thrift']
+                    f.write('thrift_port: %s\n' % port)
+
                 f.write('jmx_port: %s\n' % jmx)
                 f.close()
 
@@ -447,11 +458,16 @@ class DseNode(Node):
         with open(conf_file, 'w') as f:
             f.writelines(content)
 
-        # starting with DSE 5.0 (Spark 1.6) we need to set a unique
-        # spark.shuffle.service.port for each node
-        if self.cluster.version() > '5.0':
-            print_('Writing shuffle')
+        # set unique spark.shuffle.service.port for each node; this is only needed for DSE 5.0.x;
+        # starting in 5.1 this setting is no longer needed
+        if self.cluster.version() > '5.0' and self.cluster.version() < '5.1':
             defaults_file = os.path.join(self.get_path(), 'resources', 'spark', 'conf', 'spark-defaults.conf')
             with open(defaults_file, 'a') as f:
                 port_num = 7737 + int(node_num)
                 f.write("\nspark.shuffle.service.port %s\n" % port_num)
+
+        # create Spark working dirs; starting with DSE 5.0.10/5.1.3 these are no longer automatically created
+        for e in ["SPARK_WORKER_DIR", "SPARK_LOCAL_DIRS"]:
+            dir = env[e]
+            if not os.path.exists(dir):
+                os.makedirs(dir)
