@@ -65,7 +65,7 @@ class ToolError(Exception):
         Exception.__init__(self, message)
 
 # Groups: 1 = cf, 2 = tmp or none, 3 = suffix (Compacted or Data.db)
-_sstable_regexp = re.compile('((?P<keyspace>[^\s-]+)-(?P<cf>[^\s-]+)-)?(?P<tmp>tmp(link)?-)?(?P<version>[^\s-]+)-(?P<number>\d+)-(?P<big>big-)?(?P<suffix>[a-zA-Z]+)\.[a-zA-Z0-9]+$')
+_sstable_regexp = re.compile('((?P<keyspace>[^\s-]+)-(?P<cf>[^\s-]+)-)?(?P<tmp>tmp(link)?-)?(?P<version>[^\s-]+)-(?P<number>\d+)-(?P<format>([a-z]+)-)?(?P<suffix>[a-zA-Z]+)\.[a-zA-Z0-9]+$')
 
 
 class Node(object):
@@ -74,7 +74,7 @@ class Node(object):
     Provides interactions to a Cassandra node.
     """
 
-    def __init__(self, name, cluster, auto_bootstrap, thrift_interface, storage_interface, jmx_port, remote_debug_port, initial_token, save=True, binary_interface=None, byteman_port='0', environment_variables=None):
+    def __init__(self, name, cluster, auto_bootstrap, thrift_interface, storage_interface, jmx_port, remote_debug_port, initial_token, save=True, binary_interface=None, byteman_port='0', environment_variables=None, byteman_startup_script=None):
         """
         Create a new Node.
           - name: the name for that node
@@ -95,9 +95,11 @@ class Node(object):
         self.network_interfaces = {'thrift': common.normalize_interface(thrift_interface),
                                    'storage': common.normalize_interface(storage_interface),
                                    'binary': common.normalize_interface(binary_interface)}
+        (self.ip_addr, _) = self.network_interfaces['thrift'] if thrift_interface else self.network_interfaces['binary']
         self.jmx_port = jmx_port
         self.remote_debug_port = remote_debug_port
         self.byteman_port = byteman_port
+        self.byteman_startup_script = byteman_startup_script
         self.initial_token = initial_token
         self.pid = None
         self.data_center = None
@@ -214,6 +216,10 @@ class Node(object):
             common.validate_install_dir(self.__install_dir)
             return self.__install_dir
 
+    def node_setup(self, version, verbose):
+        dir, v = setup(version, verbose=verbose)
+        return dir
+
     def set_install_dir(self, install_dir=None, version=None, verbose=False):
         """
         Sets the path to the cassandra source directory for use by this node.
@@ -223,10 +229,9 @@ class Node(object):
             if install_dir is not None:
                 common.validate_install_dir(install_dir)
         else:
-            dir, v = setup(version, verbose=verbose)
-            self.__install_dir = dir
+            self.__install_dir = self.node_setup(version, verbose=verbose)
 
-        if self.get_base_cassandra_version() >= '4':
+        if self.get_base_cassandra_version() >= 4.0:
             self.network_interfaces['thrift'] = None
 
         self.import_config_files()
@@ -282,8 +287,8 @@ class Node(object):
         else:
             values = {
                 "commitlog_sync": "periodic",
-                "commitlog_sync_batch_window_in_ms": 10000,
-                "commitlog_sync_period_in_ms": None
+                "commitlog_sync_batch_window_in_ms": None,
+                "commitlog_sync_period_in_ms": 10000
             }
         self.set_configuration_options(values)
 
@@ -348,7 +353,7 @@ class Node(object):
             common.CASSANDRA_WIN_ENV if common.is_win() else common.CASSANDRA_ENV
         )
 
-    def grep_log(self, expr, filename='system.log'):
+    def grep_log(self, expr, filename='system.log', from_mark=None):
         """
         Returns a list of lines matching the regular expression in parameter
         in the Cassandra log of this node
@@ -356,6 +361,8 @@ class Node(object):
         matchings = []
         pattern = re.compile(expr)
         with open(os.path.join(self.get_path(), 'logs', filename)) as f:
+            if from_mark:
+                f.seek(from_mark)
             for line in f:
                 m = pattern.search(line)
                 if m:
@@ -417,7 +424,7 @@ class Node(object):
         timeouts (a TimeoutError is then raised). On successful completion,
         a list of pair (line matched, match object) is returned.
         """
-        elapsed = 0
+        start = time.time()
         tofind = [exprs] if isinstance(exprs, string_types) else exprs
         tofind = [re.compile(e) for e in tofind]
         matchings = []
@@ -429,6 +436,8 @@ class Node(object):
         output_read = False
         while not os.path.exists(log_file):
             time.sleep(.5)
+            if start + timeout < time.time():
+                raise TimeoutError(time.strftime("%d %b %Y %H:%M:%S", time.gmtime()) + " [" + self.name + "] Timed out waiting for {} to be created.".format(log_file))
             if process and not output_read:
                 process.poll()
                 if process.returncode is not None:
@@ -466,8 +475,7 @@ class Node(object):
                 else:
                     # yep, it's ugly
                     time.sleep(1)
-                    elapsed = elapsed + 1
-                    if elapsed > timeout:
+                    if start + timeout < time.time():
                         raise TimeoutError(time.strftime("%d %b %Y %H:%M:%S", time.gmtime()) + " [" + self.name + "] Missing: " + str([e.pattern for e in tofind]) + ":\n" + reads[:50] + ".....\nSee {} for remainder".format(filename))
 
                 if process:
@@ -710,10 +718,6 @@ class Node(object):
              + gently: Let Cassandra clean up and shut down properly; unless
                        false perform a 'kill -9' which shuts down faster.
         """
-        # Determine if the signal event should be updated to keep API compatibility
-        if 'gently' in kwargs and kwargs['gently'] is False:
-            signal_event = signal.SIGKILL
-
         if self.is_running():
             if wait_other_notice:
                 marks = [(node, node.mark_log()) for node in list(self.cluster.nodes.values()) if node.is_live() and node is not self]
@@ -739,6 +743,10 @@ class Node(object):
                 if self._find_pid_on_windows():
                     common.warning("Failed to terminate node: {0} with pid: {1}".format(self.name, self.pid))
             else:
+                # Determine if the signal event should be updated to keep API compatibility
+                if 'gently' in kwargs and kwargs['gently'] is False:
+                    signal_event = signal.SIGKILL
+
                 os.kill(self.pid, signal_event)
 
             if wait_other_notice:
@@ -941,6 +949,18 @@ class Node(object):
                                           'logback.xml')
         common.copy_file(new_logback_config, cassandra_conf_dir)
 
+    def update_startup_byteman_script(self, byteman_startup_script):
+        """
+        Update the byteman startup script, i.e., rule injected before the node starts.
+
+        :param byteman_startup_script: the relative path to the script
+        :raise common.LoadError: if the node does not have byteman installed
+        """
+        if self.byteman_port == '0':
+            raise common.LoadError('Byteman is not installed')
+        self.byteman_startup_script = byteman_startup_script
+        self.import_config_files()
+
     def clear(self, clear_all=False, only_data=False):
         data_dirs = ['data{0}'.format(x) for x in xrange(0, self.cluster.data_dir_count)]
         data_dirs.append("commitlogs")
@@ -967,7 +987,6 @@ class Node(object):
                 os.mkdir(dir)
 
     def run_sstable2json(self, out_file=None, keyspace=None, datafiles=None, column_families=None, keys=None, enumerate_keys=False):
-        print_("running")
         if out_file is None:
             out_file = sys.stdout
         sstable2json = self._find_cmd('sstable2json')
@@ -1047,17 +1066,16 @@ class Node(object):
         p = self.run_sstablemetadata_process(datafiles, keyspace, column_families)
         return handle_external_tool_process(p, "sstablemetadata on keyspace: {}, column_family: {}".format(keyspace, column_families))
 
-    def run_sstabledump_process(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
-        cdir = self.get_install_dir()
-        sstabledump = common.join_bin(cdir, os.path.join('tools', 'bin'), 'sstabledump')
+    def run_sstabledump_process(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False, command=False):
+        sstabledump = self._find_cmd('sstabledump')
+
         env = self.get_env()
         sstablefiles = self.__gather_sstables(datafiles=datafiles, keyspace=keyspace, columnfamilies=column_families)
         processes = []
 
-        print_(sstablefiles)
-
         def do_dump(sstable):
-            print_("-- {0} -----".format(os.path.basename(sstable)))
+            if command:
+                print_("-- {0} -----".format(os.path.basename(sstable)))
             cmd = [sstabledump, sstable]
             if enumerate_keys:
                 cmd.append('-e')
@@ -1065,15 +1083,20 @@ class Node(object):
                 for key in keys:
                     cmd = cmd + ["-k", key]
             p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, env=env)
-            processes.append(p)
+            if command:
+                out, err, rc = handle_external_tool_process(p, "sstabledump")
+                print_(out)
+                print_('\n')
+            else:
+                processes.append(p)
 
         for sstable in sstablefiles:
             do_dump(sstable)
 
         return processes
 
-    def run_sstabledump(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False):
-        processes = self.run_sstabledump_process(datafiles, keyspace, column_families, keys, enumerate_keys)
+    def run_sstabledump(self, datafiles=None, keyspace=None, column_families=None, keys=None, enumerate_keys=False, command=False):
+        processes = self.run_sstabledump_process(datafiles, keyspace, column_families, keys, enumerate_keys, command)
         results = []
 
         for p in processes:
@@ -1229,7 +1252,7 @@ class Node(object):
         elif self.get_base_cassandra_version() < 2.2:
             files = [glob.glob(os.path.join(keyspace_dir, cf_glob, "%s-%s*-Data.db" % (keyspace, column_family))) for keyspace_dir in keyspace_dirs]
         else:
-            files = [glob.glob(os.path.join(keyspace_dir, cf_glob, "*big-Data.db")) for keyspace_dir in keyspace_dirs]
+            files = [glob.glob(os.path.join(keyspace_dir, cf_glob, "*-Data.db")) for keyspace_dir in keyspace_dirs]
 
         for d in files:
             for f in d:
@@ -1240,6 +1263,34 @@ class Node(object):
     def get_sstables(self, keyspace, column_family):
         return [f for sublist in self.get_sstables_per_data_directory(keyspace, column_family) for f in sublist]
 
+    def get_sstables_via_sstableutil(self, keyspace, table, sstabletype='all', oplogs=False, cleanup=False, match='-Data.db'):
+        env = common.make_cassandra_env(self.get_install_cassandra_root(), self.get_node_cassandra_root())
+        tool_bin = self.get_tool('sstableutil')
+
+        args = [tool_bin, '--type', sstabletype]
+
+        if oplogs:
+            args.extend(['--oplog'])
+        if cleanup:
+            args.extend(['--cleanup'])
+
+        args.extend([keyspace, table])
+
+        p = subprocess.Popen(args, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+        stdout, stderr = p.communicate()
+
+        if p.returncode != 0:
+            common.error("""Error invoking sstableutil; returned {code}; args={args}; env={env}
+            stdout:
+            {stdout}
+            stderr:
+            {stderr}
+            """.format(code=p.returncode, args=args, env=env, stdout=stdout, stderr=stderr))
+            raise Exception("Error invoking sstableutil; returned {code}".format(code=p.returncode))
+
+        return sorted(filter(lambda s: match in s, stdout.splitlines()))
+
     def stress_process(self, stress_options=None, whitelist=False):
         if stress_options is None:
             stress_options = []
@@ -1291,11 +1342,19 @@ class Node(object):
         output = self.nodetool('info')[0]
         return _get_load_from_info_output(output)
 
-    def flush(self):
-        self.nodetool("flush")
+    def flush(self, options=None):
+        if options is None:
+            options = []
+        args = ["flush"] + options
+        cmd = ' '.join(args)
+        self.nodetool(cmd)
 
-    def compact(self):
-        self.nodetool("compact")
+    def compact(self, options=None):
+        if options is None:
+            options = []
+        args = ["compact"] + options
+        cmd = ' '.join(args)
+        self.nodetool(cmd)
 
     def drain(self, block_on_log=False):
         mark = self.mark_log()
@@ -1313,8 +1372,12 @@ class Node(object):
     def move(self, new_token):
         self.nodetool("move " + str(new_token))
 
-    def cleanup(self):
-        self.nodetool("cleanup")
+    def cleanup(self, options=None):
+        if options is None:
+            options = []
+        args = ["cleanup"] + options
+        cmd = ' '.join(args)
+        self.nodetool(cmd)
 
     def decommission(self, force=False):
         cmd = 'decommission'
@@ -1602,6 +1665,8 @@ class Node(object):
         if self.byteman_port != '0':
             byteman_jar = glob.glob(os.path.join(self.get_install_dir(), 'build', 'lib', 'jars', 'byteman-[0-9]*.jar'))[0]
             agent_string = "-javaagent:{}=listener:true,boot:{},port:{}".format(byteman_jar, byteman_jar, str(self.byteman_port))
+            if self.byteman_startup_script is not None:
+                agent_string = agent_string + ",script:{}".format(self.byteman_startup_script)
             if common.is_modern_windows_install(self.get_base_cassandra_version()):
                 with open(conf_file, "r+") as conf_rewrite:
                     conf_lines = conf_rewrite.readlines()
@@ -1971,8 +2036,11 @@ def _get_load_from_info_output(info):
 
 
 def _grep_log_for_errors(log):
+    except_re = re.compile(r'[Ee]xception|AssertionError')
+    log_cat_re = re.compile(r'(INFO|DEBUG|WARN|ERROR)')
+
     def log_line_category(line):
-        match = re.search(r'(INFO|DEBUG|WARN|ERROR)', line)
+        match = log_cat_re.search(line)
         return match.group(0) if match else None
 
     matches = []
@@ -1980,12 +2048,22 @@ def _grep_log_for_errors(log):
     loglines = log.splitlines()
 
     for line_num, line in enumerate(loglines):
-        if log_line_category(line) == 'ERROR':
+        found_exception = False
+        line_category = log_line_category(line)
+        if line_category == 'ERROR':
             matches.append([line])
+            found_exception = True
 
+        elif line_category == 'WARN':
+            match = except_re.search(line)
+            if match is not None:
+                matches.append([line])
+                found_exception = True
+
+        if found_exception:
             for next_line_num in range(line_num + 1, len(loglines)):
                 next_line = loglines[next_line_num]
-                # if a log line can't be identified, assume continuation of an ERROR message
+                # if a log line can't be identified, assume continuation of an ERROR/WARN exception
                 if log_line_category(next_line) is None:
                     matches[-1].append(next_line)
                 else:
